# Install and Load the Required Libraries
#install.packages(c("WDI", "dplyr"))
library(WDI)
library(dplyr)

# Define the file path for the cached population data
cache_file <- "population_data.csv"

# Function to fetch and cache population data
fetch_population_data <- function() {
  # Fetch population data for the year 2022
  population_data <- WDI(country = "all", indicator = "SP.POP.TOTL", start = 2022, end = 2022)
  population_data <- population_data %>%
    select(iso2c, country, SP.POP.TOTL) %>%
    rename(population = SP.POP.TOTL)
  
  # Save the fetched data to a CSV file
  write.csv(population_data, file = cache_file, row.names = FALSE)
  return(population_data)
}

# Check if the cached file exists
if (file.exists(cache_file)) {
  # Read the cached population data
  population_data <- read.csv(cache_file, stringsAsFactors = FALSE)
} else {
  # Fetch and cache the population data
  population_data <- fetch_population_data()
}

# Read the Developers' Data from the CSV File
developers_data <- read.csv("dd-by-country.csv", stringsAsFactors = FALSE)

# Display column names to verify
print(colnames(developers_data))

# Ensure column names match expected names
colnames(developers_data) <- c("number_of_developers", "country_code")

# Display column names to verify
print(colnames(population_data))

# Merge the Data
merged_data <- developers_data %>%
  left_join(population_data, by = c("country_code" = "iso2c"))

# Calculate the Incidence of Debian Developers per 100,000 Inhabitants
merged_data <- merged_data %>%
  mutate(incidence = (number_of_developers / population) * 100000)

# Sort the Data by Incidence in Descending Order
merged_data <- merged_data %>%
  arrange(desc(incidence))

# Save the Result to CSV
write.csv(merged_data, file = "debian_developers_incidence.csv", row.names = FALSE)

# Print a message indicating completion
print("Incidence calculation completed, sorted by incidence, and saved to debian_developers_incidence.csv")
